/*
	(C) 1995-96 AROS - The Amiga Research OS
	$Id: activatewindow.c,v 1.15 2021/10/10 01:57:13 jacadcaps Exp $

	Desc:
	Lang: english
*/
#include <proto/graphics.h>
#include <proto/timer.h>
#include <proto/layers.h>
#include <intuition/gadgetclass.h>
#include "intuition_intern.h"
#include "inputhandler.h"
#include "inputhandler_actions.h"
#include "inputhandler_support.h"
#include "boopsigadgets.h"
#include "boolgadgets.h"
#include "propgadgets.h"
#include "strgadgets.h"
#include "gadgets.h"
#include "menus.h"
#include "mosmisc.h"
#include "layermanager.h"
#include <intuition/pointerclass.h>

#define	DEBUG_POINTER(x)	;

/*****************************************************************************

	NAME */
#include <proto/intuition.h>

AROS_LH1(void, ActivateWindow,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Window *, window, A0),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 75, Intuition)

/*  FUNCTION
	Activates the specified window. The window gets the focus
	and all further input it sent to that window. If the window
	requested it, it will get a IDCMP_ACTIVEWINDOW message.

	INPUTS
	window - The window to activate

	RESULT
	None.

	NOTES
	If the user has an autopointer tool (sunmouse), the call will
	succeed, but the tool will deactivate the window right after
	this function has activated it. It is no good idea to try to
	prevent this by waiting for IDCMP_INACTIVEWINDOW and activating
	the window again since that will produce an anoying flicker and
	it will slow down the computer a lot.

	EXAMPLE

	BUGS

	SEE ALSO
	ModiyIDCMP(), OpenWindow(), CloseWindow()

	INTERNALS

	HISTORY
	29-10-95    digulla automatically created from
			    intuition_lib.fd and clib/intuition_protos.h

*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

	struct ActivateWindowActionMsg msg;

	DEBUG_ACTIVATEWINDOW(dprintf("ActivateWindow: Window 0x%lx\n",
				     window));

	if (IsDuringSelfResize(IntuitionBase)) return;

	msg.window = window;
	msg.useLayerManager = TRUE;
	msg.allowAnimations = TRUE;
	DoASyncAction((APTR)int_activatewindow, &msg.msg, sizeof(msg), IntuitionBase);

	AROS_LIBFUNC_EXIT

} /* ActivateWindow */

/* This is called on the input.device's context */

VOID int_activatewindow(struct ActivateWindowActionMsg *msg,
						struct IntuitionBase *IntuitionBase)
{
	struct IIHData *iihdata = (struct IIHData *)IBase->InputHandler->is_Data;

	/* On the Amiga ActivateWindow is delayed if there
	   is an active gadget (altough this does not seem
	   to be true for string gadgets). We just ignore
	   ActivateWindow in such a case. */

	struct Window *window = msg->window;
	ULONG lock = 0;
	struct Window *oldactive;
	#ifndef SKINS
	Object *pointer = NULL;
	#endif
	struct IntScreen *scr = NULL;
	struct InputEvent *ie;
	BOOL toolbar = ((window) ? ((window->MoreFlags & WMFLG_TOOLBARWINDOW) ? TRUE : FALSE) : FALSE);

	if (iihdata && ((!(iihdata->ActiveGadget)) || (iihdata->ActiveGadget && ((iihdata->ActiveGadget->GadgetType & GTYP_SYSTYPEMASK) == GTYP_SDEPTH))) &&
		(!toolbar) &&
		((!window) || (!(IBase->IControlPrefs.ic_Flags & ICF_SCREENACTIVATION)) || ((IBase->IControlPrefs.ic_Flags & ICF_SCREENACTIVATION) && (window->WScreen == IntuitionBase->FirstScreen))) &&
		((!window) || IsLayerHitable(WLAYER(window))) &&
		((!IntuitionBase->ActiveWindow) || (IntuitionBase->ActiveWindow != IS(IntuitionBase->FirstScreen)->EventOwnerWindow)))
	{

		if (!ILOCKCHECK(((struct IntuiActionMsg *)msg))) lock = LockIBase(0UL);

		oldactive = IntuitionBase->ActiveWindow;
		IntuitionBase->ActiveWindow = window;

		DEBUG_ACTIVATEWINDOW(dprintf("IntActivateWindow: Window 0x%lx OldActive 0x%lx\n",
					     window, oldactive));

		if (oldactive != window)
		{
			IBase->PointerDelay = 0;

			if (oldactive)
			{
				#ifdef __MORPHOS__
				#include "workaround.activatewindow.c"
				#endif

				ih_fire_intuimessage(oldactive,
						     IDCMP_INACTIVEWINDOW,
						     0,
						     oldactive,
						     IntuitionBase);
				scr = IS(oldactive->WScreen);
				scr->Screen.Title = scr->Screen.DefaultTitle;
			}

			if (window)
			{
				/* App task is allowed to modify window->Flags, for example
				   set/clear WFLG_RMBTRAP. It is noid said that every compiler
				   on every machine produces an atomic instruction, so some
				   kind of semaphore is needed */

				Forbid();
				window->Flags |= WFLG_WINDOWACTIVE;
				Permit();

#if 0
				pointer = IW(window)->pointer;
				if (IW(window)->busy)
#ifdef SKINS
					pointer = IS(window->WScreen)->IMonitorNode->Pointers[POINTERTYPE_BUSY];
#else
					pointer = IBase->BusyPointer;
#endif
#endif
				scr = IS(window->WScreen);
				if (window->ScreenTitle)
					scr->Screen.Title = window->ScreenTitle;

#ifdef SKINS
				if (IW(window)->pointer_delay)
				{
					IW(window)->pointer_delay = 0;
					IW(window)->pointer = IW(window)->pointer_delay_pointer;
					IW(window)->pointertype = IW(window)->pointer_delay_pointertype;
					IW(window)->busy = IW(window)->pointer_delay_busy;
				}
#endif
			}

			/* now set the ActiveScreen! */
			if (scr)
			{
				struct Screen *old;
				struct Window *win;

				old = IntuitionBase->ActiveScreen;

				if (old && (old != (struct Screen *)scr))
				{

					/* this avoids mouse "jump" effect */
					scr->Screen.MouseX = old->MouseX;
					scr->Screen.MouseY = old->MouseY;

					win = scr->Screen.FirstWindow;

					while (win)
					{
						UpdateMouseCoords(win);
						win = win->NextWindow;
					}
#ifdef SKINS
					UpdateScreenPointer(old,IntuitionBase);
#endif
				}

				IntuitionBase->ActiveScreen = (struct Screen *)scr;

			}

#ifdef SKINS
			if (scr)
			{
				CheckMouseZone(FALSE,IntuitionBase);
				UpdateScreenPointer(&scr->Screen,IntuitionBase);
			}
#else
			if (scr)
			{
				struct SharedPointer *shared_pointer;
				//ULONG tmp;

				//works when window == NULL
				pointer = GetWindowPointer(window,IntuitionBase);

				if (!pointer)
					pointer = IBase->DefaultPointer;

				GetAttr(POINTERA_SharedPointer, pointer, (IPTR *)&shared_pointer);

				DEBUG_POINTER(dprintf("ActivateWindow: scr 0x%lx pointer 0x%lx sprite 0x%lx\n",
						      scr, pointer, shared_pointer->sprite));

				if (ChangeExtSpriteA(&scr->Screen.ViewPort,
						     scr->Pointer->sprite, shared_pointer->sprite, NULL))
				{
					//jDc: use NL functions here - they don't call LockIBase()
					ObtainSharedPointerNL(shared_pointer, IntuitionBase);
					ReleaseSharedPointerNL(scr->Pointer, IntuitionBase);
					scr->Pointer = shared_pointer;
					if (window)
					{
						DEBUG_POINTER(dprintf("ActivateWindow: Old window XOffset %ld YOffset %ld\n",
								      window->XOffset,
								      window->YOffset));

						window->XOffset = shared_pointer->xoffset;
						window->YOffset = shared_pointer->yoffset;

						DEBUG_POINTER(dprintf("ActivateWindow: New window XOffset %ld YOffset %ld\n",
								      window->XOffset,
								      window->YOffset));

					}

					SetPointerPos(scr->Screen.MouseX, scr->Screen.MouseY);
				}
				else
				{
					DEBUG_POINTER(dprintf("ActivateWindow: can't set pointer.\n"));
				}
			}
#endif /* SKINS */
		}

		if (!ILOCKCHECK(((struct IntuiActionMsg *)msg))) UnlockIBase(lock);

		if (window && msg->useLayerManager &&
			(WMFLG_LAYERMANAGER == (window->MoreFlags & WMFLG_LAYERMANAGER)) && 
			(0 == (window->MoreFlags & WMFLG_OVERLAYATTACHED)) &&
			oldactive != window)
		{
			windowLayersActivated(window, msg->allowAnimations, IntuitionBase);
		}

		if (oldactive && oldactive != window)
		{
			Forbid();
			oldactive->Flags &= ~WFLG_WINDOWACTIVE;
			Permit();

			if ((((struct Library *)LayersBase)->lib_Version >= 52) &&
				(!(oldactive->Flags & WFLG_BORDERLESS)))
			{
				LayersControlTags(&oldactive->WScreen->LayerInfo,
					LT_Layer,(ULONG)BLAYER(oldactive) ? (ULONG)BLAYER(oldactive) : (ULONG)WLAYER(oldactive),
					LT_ActiveEffect,FALSE,
					TAG_DONE);
			}

			int_refreshwindowframe(oldactive, REFRESHGAD_BORDER, 0, IntuitionBase);
			if (!window || oldactive->WScreen != window->WScreen)
				RenderScreenBar(oldactive->WScreen, FALSE, IntuitionBase);

			if ((WMFLG_LAYERMANAGER == (oldactive->MoreFlags & WMFLG_LAYERMANAGER)) && (0 == (oldactive->MoreFlags & WMFLG_OVERLAYATTACHED)))
			{
				windowLayersDeactivated(oldactive, IntuitionBase);
			}
		}

		if (window && oldactive != window)
		{
			if ((((struct Library *)LayersBase)->lib_Version >= 52) &&
				(!(window->Flags & WFLG_BORDERLESS)))
			{
				LayersControlTags(&window->WScreen->LayerInfo,
					LT_Layer,(ULONG)BLAYER(window) ? (ULONG)BLAYER(window) : (ULONG)WLAYER(window),
					LT_ActiveEffect,TRUE,
					TAG_DONE);
			}

			int_refreshwindowframe(window, REFRESHGAD_BORDER, 0, IntuitionBase);
			RenderScreenBar(window->WScreen, FALSE, IntuitionBase);
		}

#ifdef TIMEVALWINDOWACTIVATION
		if (window)
		{
			GetSysTime(&IW(window)->activationtime);
		}
#endif

		if ((ie = AllocInputEvent(iihdata)))
		{
			ie->ie_Class = IECLASS_EVENT;
			ie->ie_Code = IECODE_NEWACTIVE;
			ie->ie_EventAddress = window;
			CurrentTime(&ie->ie_TimeStamp.tv_secs, &ie->ie_TimeStamp.tv_micro);
		}

		if (window && oldactive != window)
		{
			ih_fire_intuimessage(window,
					     IDCMP_ACTIVEWINDOW,
					     0,
					     window,
					     IntuitionBase);
		}
	} else {
		if (window)
		{
			int_refreshwindowframe(window, REFRESHGAD_BORDER, 0, IntuitionBase);

			//jDc: I guess this makes sense since the app's internal activity state will get
			//updated properly.

			ih_fire_intuimessage(window,
						 IDCMP_INACTIVEWINDOW,
					     0,
					     window,
					     IntuitionBase);

			UpdateScreenPointer(window->WScreen,IntuitionBase);
		}
	}

#ifdef __MORPHOS__
	CrashCheck(window,IntuitionBase);
#endif

}

